@@ -16,10 +16,10 @@
 .newDatashieldSession <- function(opal, restore=NULL) {
   query <- list()
   if (!is.null(restore)) {
-    query <- list(restore=restore)  
+    query$restore <- restore
   }
   if (!is.null(opal$profile) && !is.na(opal$profile)) {
-    query <- list(profile=opal$profile)  
+    query$profile <- opal$profile
   }
   res <- opalr::opal.post(opal, "datashield", "sessions", query=query)
   return(res$id)
